import Hero from "./Hero";

const { ccclass, property } = cc._decorator;

@ccclass
export default class Helloworld extends cc.Component {

    @property(cc.Prefab)
    heroPrefab: cc.Prefab[] = [];
    @property(cc.Prefab)
    buildingPrefab: cc.Prefab[] = [];

    @property(cc.Node)
    roleLayer: cc.Node = null;
    @property(cc.Node)
    floorLayer: cc.Node = null;

    @property(cc.Label)
    lab: cc.Label = null;
    @property(cc.Label)
    time: cc.Label = null;

    private _count: number = 0;
    private _index: number = 0;
    private _date: Date = null;
    private building: Hero[] = [];

    public get count() {
        return this._count;
    }

    public set count(v: number) {
        this._count = v;
        this.lab.string = this.count.toString();
    }

    onLoad() {
        // 如果不注释下面创建建筑的情况， 6秒就崩溃
        this.initBuilding();

        // 5个英雄角色第6秒必崩，少于5个就不会那么快崩溃了会出现纹理出错
        for (let index = 0; index < 5; index++) {
            let hero = this.addHero();
            let node = hero ? hero.node : null;
            if (node) {
                node.x = -100;
                node.y = index * 100;
                node.zIndex = -index;
            }
        }

        this._date = new Date();
    }

    protected initBuilding() {
        for (let index = 0; index < this.buildingPrefab.length; index++) {
            let nameIndex = index;
            const prefab = this.buildingPrefab[index];

            let node = cc.instantiate(prefab);
            node.scaleX = -1;
            this.roleLayer.addChild(node);
            node.y = nameIndex * 100;
            node.zIndex = -nameIndex;

            let heroComponent = node.getComponent(Hero);
            heroComponent.runAction("idle");
            this.building.push(heroComponent);

            this.scheduleOnce(() => {
                this.changeBuildingAction(nameIndex);
             }, 1 + nameIndex * 1);

            this.scheduleOnce(() => {
                this.changeBuildingAction(nameIndex);
            }, 2 + nameIndex * 1);

            this.scheduleOnce(() => {
                this.changeBuildingAction(nameIndex);
            }, 3 + nameIndex * 1);
        }
    }

    protected changeBuildingAction(index: number) {
        let building = this.building[index];
        if (building) {

            switch (building.runActionName) {
                case "idle":
                    building.runAction("wall");
                    break;
                case "wall":
                    building.runAction("collapse");
                    break;
                case "collapse":
                    this.roleLayer.removeChild(building.node);
                    this.floorLayer.addChild(building.node);
                    break;
                default:
                    break;
            }
        }
    }

    protected addHero(): Hero {
        if (this.heroPrefab == null) return;

        let prefab = this.heroPrefab[this._index++];
        let hero = cc.instantiate(prefab);
        let heroComponent = hero.getComponent(Hero);
        heroComponent.diedFunction = (node: cc.Node) => {
            this.removeHero(heroComponent);
        }

        this.roleLayer.addChild(hero);
        heroComponent.runAction();
        this._index %= this.heroPrefab.length;
        this.count++;

        return heroComponent;
    }

    lateUpdate() {
        let date = new Date();
        let value = date.getTime() - this._date.getTime();
        this.time.string = Math.floor(value / 1000).toString();
    }

    public removeHero(hero: Hero) {
        this.count--;

        // 由于我们会提前做一些创建缓存处理，像cc.NodePool会使用下面的方法，
        // 不使用这个方法就不会崩溃，处理一下谢谢
        hero.node.removeFromParent(true);

        // 使用removeChild也会崩溃
        // this.roleLayer.removeChild(hero.node);

        this.floorLayer.addChild(hero.node);

        // 模拟一定时间后使用回收利用角色资源
        this.scheduleOnce(() => {
            hero.node.removeFromParent(true);

            // 使用removeChild也会崩溃
            // this.floorLayer.removeChild(hero.node);

            this.roleLayer.addChild(hero.node);

            hero.runAction();
            this.count++;

        }, 2);
    }

    public onClickBtn() {
        cc.director.loadScene(`helloworld`);
    }
}
